import os
import sys
import re
from optparse import OptionParser, OptionGroup, OptionValueError

import mice
from helpers import (usb_device_is_connected, find_hidraw_device_path,
        is_color, choices_to_list, choices_to_string)
from rival_mouse import RivalMouse
from version import VERSION


DEBUG_DRY = "DEBUG_DRY" in os.environ
DEBUG_VENDOR_ID = None
DEBUG_PRODUCT_ID = None
if "DEBUG_MOUSE" in os.environ:
    DEBUG_VENDOR_ID = os.environ["DEBUG_MOUSE"].split(":")[0]
    DEBUG_PRODUCT_ID = os.environ["DEBUG_MOUSE"].split(":")[1]
DEBUG = DEBUG_DRY or DEBUG_VENDOR_ID


def get_plugged_mouse_profile():
    """Returns the profile of the mouse plugged on the computer."""
    for profile in mice.mice_list:
        if profile["vendor_id"] == DEBUG_VENDOR_ID and profile["product_id"] == DEBUG_PRODUCT_ID:
            return profile
        elif not DEBUG_VENDOR_ID and usb_device_is_connected(profile["vendor_id"], profile["product_id"]):
            return profile


def _print_compatible_mice():
    """Prints mice currently supported by this software."""
    print("\n".join([profile["name"] for profile in mice.mice_list]))


def _check_color(option, opt_str, value, parser):
    """OptionParser callback to check if the given color is valid."""
    if not is_color(value):
        raise OptionValueError("option %s: invalid color: '%s'" % (opt_str, value))
    setattr(parser.values, option.dest, value)


def _generate_default_cli_options(parser):
    """Add common cli options"""
    parser.add_option("-l", "--list",
        help="print compatible mice and exit",
        action="store_true"
        )


def _generate_mouse_cli_options(parser, profile):
    """generate CLI specific to the plugged mouse."""
    group = OptionGroup(parser, "%s Options" % profile["name"])
    commands = sorted(profile["commands"].keys());
    for command in commands:
        cmd = profile["commands"][command]
        if not cmd["cli"]:
            continue
        if cmd["value_type"] == "choice":
            description = "%s (values: %s, default: %s)" % (
                    cmd["description"],
                    choices_to_string(cmd["choices"]),
                    str(cmd["default"])
                    )
            group.add_option(
                    *cmd["cli"],
                    help=description,
                    dest=command,
                    choices=choices_to_list(cmd["choices"])
                    )
        elif cmd["value_type"] == "rgbcolor":
            description = "%s (e.g. red, #ff0000, ff0000, #f00, f00, default: %s)" % (
                    cmd["description"],
                    str(cmd["default"])
                    )
            group.add_option(
                    *cmd["cli"],
                    dest=command,
                    help=description,
                    type="string",
                    action="callback",
                    callback=_check_color
                    )
        elif cmd["value_type"] == "range":
            description = "%s (from %i to %i in increments of %i, default: %i)" % (
                    cmd["description"],
                    cmd["range_min"],
                    cmd["range_max"],
                    cmd["range_increment"],
                    cmd["default"]
                    )
            group.add_option(
                    *cmd["cli"],
                    dest=command,
                    help=description
                    # choices=[str(i) for i in range(cmd["range_min"], cmd["range_max"] + 1, cmd["range_increment"])]
                    )
        else:
            raise NotImplementedError("Cannot generate CLI option for value_type '%s'" % cmd["value_type"])
    group.add_option("-r", "--reset",
            help="Reset all options to their factory values",
            action="store_true"
            )
    parser.add_option_group(group)


def main():
    """Run rivalcfg's CLI"""
    # Find the plugged mouse's profile
    profile = get_plugged_mouse_profile()

    # Debug infos
    if DEBUG:
        print("[DEBUG] Debugging rivalcfg %s..." % VERSION)
    if DEBUG_DRY:
        print("[DEBUG] Dry run enabled")
    if DEBUG_PRODUCT_ID:
        print("[DEBUG] Debugging mouse profile %s:%s" % (DEBUG_VENDOR_ID, DEBUG_PRODUCT_ID))
    if DEBUG and profile:
        print("[DEBUG] Mouse profile found: %s" % profile["name"])
    if DEBUG and not profile:
        print("[DEBUG] No mouse profile found")

    # Generates CLI options
    parser = OptionParser("Usage: rivalcfg [options]",
            version=VERSION,
            epilog="Please report any bug on Github: https://github.com/flozz/rivalcfg/issues"
            )
    _generate_default_cli_options(parser);
    if profile:
        _generate_mouse_cli_options(parser, profile)

    options, args = parser.parse_args();

    # Mouse indep commands
    if options.list:
        _print_compatible_mice()
        sys.exit(0)

    # Check everything is ok
    if not profile:
        print("E: No compatible mouse found. Type 'rivalcfg --help' for more informations.")
        sys.exit(1);

    if not DEBUG_DRY and not find_hidraw_device_path(profile["vendor_id"], profile["product_id"], profile["hidraw_interface_number"]):
        print("E: The '%s' mouse is plugged in but the control interface is not available." % profile["name"])
        print("\nTry to:")
        print("  * unplug the mouse from the USB port,")
        print("  * wait fiew seconds,")
        print("  * and plug the mouse to the USB port again.")
        sys.exit(1)

    if len(sys.argv) <= 1:
        parser.print_help()
        sys.exit(1)

    # Configure the mouse
    mouse = RivalMouse(profile)

    if profile["command_before"]:
        getattr(mouse, profile["command_before"])()

    if hasattr(options, "reset") and options.reset:
        mouse.set_default()

    for option, value in options.__dict__.items():
        if option in ["list", "reset"] or value == None:
            continue
        if value.isdigit():
            value = int(value)
        getattr(mouse, option)(value)

    if profile["command_after"]:
        getattr(mouse, profile["command_after"])()




rival100 = {
    "name": "SteelSeries Rival 100",

    "vendor_id": "1038",
    "product_id": "1702",
    "hidraw_interface_number": 0,

    "command_before": None,
    "command_after": "save",

    "commands": {

        "set_sensitivity1": {
            "description": "Set sensitivity preset 1",
            "cli": ["-s", "--sensitivity1"],
            "command": [0x03, 0x01],
            "value_type": "choice",
            "choices": {
                250: 0x08,
                500: 0x07,
                1000: 0x06,
                1250: 0x05,
                1500: 0x04,
                1750: 0x03,
                2000: 0x02,
                4000: 0x01,
            },
            "default": 1000,
        },

        "set_sensitivity2": {
            "description": "Set sensitivity preset 2",
            "cli": ["-S", "--sensitivity2"],
            "command": [0x03, 0x02],
            "value_type": "choice",
            "choices": {
                250: 0x08,
                500: 0x07,
                1000: 0x06,
                1250: 0x05,
                1500: 0x04,
                1750: 0x03,
                2000: 0x02,
                4000: 0x01,
            },
            "default": 2000,
        },

        "set_polling_rate": {
            "description": "Set polling rate in Hz",
            "cli": ["-p", "--polling-rate"],
            "command": [0x04, 0x00],
            "value_type": "choice",
            "choices": {
                125: 0x04,
                250: 0x03,
                500: 0x02,
                1000: 0x01,
            },
            "default": 1000,
        },

        "set_color": {
            "description": "Set the mouse backlight color",
            "cli": ["-c", "--color"],
            "command": [0x05, 0x00],
            "value_type": "rgbcolor",
            "default": "#00FFFF"
        },

        "set_light_effect": {
            "description": "Set the light effect",
            "cli": ["-e", "--light-effect"],
            "command": [0x07, 0x00],
            "value_type": "choice",
            "choices": {
                "steady": 0x01,
                "breath": 0x03,
                1: 0x01,
                2: 0x02,
                3: 0x03,
                4: 0x04,
            },
            "default": "steady",
        },

        "set_btn6_action": {
            "description": "Set the action of the button under the wheel",
            "cli": ["-b", "--btn6-action"],
            "command": [0x0B],
            "value_type": "choice",
            "choices": {
                "default": 0x00,
                "os": 0x01,
            },
            "default": "default",
        },

        "save": {
            "description": "Save the configuration to the mouse memory",
            "cli": None,
            "command": [0x09, 0x00],
            "value_type": None,
        },

    },

}



import os
from functools import partial

from helpers import (
        find_hidraw_device_path, is_color, color_string_to_rgb,
        choices_to_string, merge_bytes)


DEBUG_DRY = "DEBUG_DRY" in os.environ


class RivalMouse:

    """Generic class to handle any Rival mouse."""

    def __init__(self, profile):
        """Contructor.

        Arguments:
        profile -- the mouse profile (rivalcfg.mice.*)
        """
        self.profile = profile
        self.device_path = None
        self._device = None
        if not DEBUG_DRY:
            self._device_find()
            self._device_open()

    def set_default(self):
        """Set all option to their factory values."""
        for command in self.profile["commands"]:
            if "default" in self.profile["commands"][command]:
                getattr(self, command)(self.profile["commands"][command]["default"])

    def _device_find(self):
        """Find the HIDRAW device file path."""
        self.device_path = find_hidraw_device_path(self.profile["vendor_id"],
                self.profile["product_id"], self.profile["hidraw_interface_number"])
        if not self.device_path:
            raise Exception("Unable to locate the HIDRAW interface for the given profile")

    def _device_open(self):
        """Open the device file"""
        self._device = open(self.device_path, "wb")

    def _device_write(self, *bytes_):
        """Write bytes to the device file.

        Arguments:
        *bytes_ -- bytes to write
        """
        if DEBUG_DRY:
            print("[DEBUG] _device_write: %s" % " ".join(["%02X" % b for b in bytes_]))
            return
        if not self._device:
            return;
        self._device.write(bytearray(bytes_))
        self._device.flush()

    def _device_close(self):
        """Close the device file."""
        if self._device:
            self._device.close()
            self._device = None

    def _handler_choice(self, command, value):
        """Handle commands with value picked from a dict."""
        if not value in command["choices"]:
            raise ValueError("value must be one of [%s]" % choices_to_string(command["choices"]))
        if "value_transform" in command:
            self._device_write(*merge_bytes(command["command"], command["value_transform"](value)))
        else:
            self._device_write(*merge_bytes(command["command"], command["choices"][value]))

    def _handler_rgbcolor(self, command, *args):
        """Handle commands with RGB color values."""
        color = (0x00, 0x00, 0x00)
        if len(args) == 3:
            for value in args:
                if type(value) != int or value < 0 or value > 255:
                    raise ValueError()
            color = args
        elif len(args) == 1 and type(args[0]) == str and is_color(args[0]):
            color = color_string_to_rgb(args[0])
        else:
            raise ValueError()
        if "value_transform" in command:
            self._device_write(*merge_bytes(command["command"], command["value_transform"](*args)))
        else:
            self._device_write(*merge_bytes(command["command"], color))

    def _handler_range(self, command, value):
        """Handle commands with value from a range."""
        if not command["range_min"] <= value <= command["range_max"]:
            raise ValueError("Value %i not in range (%i, %i)" % (
                value,
                command["range_min"],
                command["range_max"]
                ))
        if value % command["range_increment"] != 0:
            raise ValueError("Value %i is not an increment of %i" % (
                value,
                command["range_increment"]
                ))
        if "value_transform" in command:
            self._device_write(*merge_bytes(command["command"], command["value_transform"](value)))
        else:
            self._device_write(*merge_bytes(command["command"], value))


    def _handler_none(self, command):
        """Handle commands with no values."""
        self._device_write(*command["command"])

    def __getattr__(self, name):
        if not name in self.profile["commands"]:
            raise AttributeError("There is no command named '%s'" % name)
        command = self.profile["commands"][name]
        handler = "_handler_%s" % str(command["value_type"]).lower()
        if not hasattr(self, handler):
            raise Exception("There is not handler for the '%s' value type" % command["value_type"])
        return partial(getattr(self, handler), command)

    def __repr__(self):
        return "<RivalMouse %s (%s:%s) at %s>" % (
                self.profile["name"],
                self.profile["vendor_id"],
                self.profile["product_id"],
                self.device_path
                )

    def __str__(self):
        return self.__repr__()

    def __del__(self):
        self._device_close()




#!/usr/bin/env python
# encoding: UTF-8

from setuptools import setup, find_packages
from rivalcfg import VERSION

setup(
    name="rivalcfg",
    version=VERSION,
    description="Configure SteelSeries Rival gaming mice",
    url="https://github.com/flozz/rivalcfg",
    license="WTFPL",

    long_description="""
    rivalcfg is a small CLI utility program that allows you to configure
    SteelSeries Rival gaming mice on Linux.

    Supported mice:

    * SteelSeries Rival (experimental¹)
    * SteelSeries Rival 100
    * ~~SteelSeries Rival 300~~ **WORK IN PROGRESS**

    __experimental¹:__ I don't have this mouse so I am unable to test it. If you
    have this mouse, please test all commands and report what is working or not by
    openning an issue on Github: https://github.com/flozz/rivalcfg/issues


    Usage: rivalcfg [options]

    Type "rivalcfg --help" to list available options.
    """,

    author="Fabien LOISON",
    author_email="http://www.flozz.fr/",

    keywords="steelseries rival rival100 mouse",
    platforms=["Linux"],

    packages=find_packages(),

    install_requires=[
        "pyudev>=0.19.0"
    ],

    entry_points = {
        "console_scripts": [
            "rivalcfg = rivalcfg.cli:main"
        ]
    },
)

